import cherrypy

from landlab_rest import create_app

app = create_app()


def start(host, port):

    # Mount the application
    cherrypy.tree.graft(app, "/")

    # Unsubscribe the default server
    cherrypy.server.unsubscribe()

    # Instantiate a new server object
    server = cherrypy._cpserver.Server()

    # Configure the server object
    server.socket_host = host
    server.socket_port = port
    server.thread_pool = 30

    # For SSL Support
    # server.ssl_module            = 'pyopenssl'
    # server.ssl_certificate       = 'ssl/certificate.crt'
    # server.ssl_private_key       = 'ssl/private.key'
    # server.ssl_certificate_chain = 'ssl/bundle.crt'

    # Subscribe this server
    server.subscribe()

    # Start the server engine (Option 1 *and* 2)

    cherrypy.engine.start()
    cherrypy.engine.block()



# -*- coding: utf-8 -*-
import click

from .start import start


@click.command()
@click.version_option(prog_name="landlab-sketchbook")
@click.option("-p", "--port", default=80, help="port to run on")
@click.option("--host", default="0.0.0.0", help="host IP address")
@click.option("--silent", is_flag=True, help="only emit messages on error")
def main(host, port, silent):
    if not silent:
        click.secho("🚀 launching landlab sketchbook on {0}:{1}".format(host, port))
    start(host, port)



import importlib

from flask import Blueprint, Flask, jsonify, url_for


def register_blueprints(app):
    rv = []

    for name in ["graphs"]:
        m = importlib.import_module(".api.{bp}".format(bp=name), package="landlab_rest")
        for item in dir(m):
            item = getattr(m, item)
            if isinstance(item, Blueprint):
                app.register_blueprint(item, url_prefix="/" + item.name)
            rv.append(item)

    return rv


def create_app():
    app = Flask(__name__, instance_relative_config=True)

    @app.route("/")
    def site_map():
        COLLECTIONS = ["graphs"]

        map = {"@type": "api", "href": url_for(".site_map")}
        links = []
        for rel in COLLECTIONS:
            href = url_for(".".join([rel, "show"]))
            links.append({"rel": rel, "href": href})
        map["links"] = links
        return jsonify(map)

    register_blueprints(app)

    return app



import json
import urllib

import landlab
from flask import Blueprint, Response, jsonify, request

graphs_page = Blueprint("graphs", __name__)


def as_resource(resp):
    return Response(
        json.dumps(resp, sort_keys=True, indent=2, separators=(",", ": ")),
        mimetype="application/x-resource+json; charset=utf-8",
    )


def as_collection(resp):
    return Response(
        json.dumps(resp, sort_keys=True, indent=2, separators=(",", ": ")),
        mimetype="application/x-collection+json; charset=utf-8",
    )


def jsonify_collection(items):
    collection = []
    for item in items:
        collection.append(item.to_resource())
    return Response(
        json.dumps(collection, sort_keys=True, indent=2, separators=(",", ": ")),
        mimetype="application/x-collection+json; charset=utf-8",
    )


def to_resource(grid, href=None, repr_=None):
    return {"_type": "graph", "href": href, "graph": grid_as_dict(grid), "repr": repr_}


def grid_as_dict(grid):
    return grid.ds.to_dict()


@graphs_page.route("/")
def show():
    graphs = ["hex", "raster", "radial"]
    graphs.sort()
    return jsonify(graphs)


@graphs_page.route("/raster")
def raster():
    args = dict(
        shape=request.args.get("shape", "3,3"),
        spacing=request.args.get("spacing", "1.0,1.0"),
        origin=request.args.get("origin", "0.0,0.0"),
    )

    shape = tuple(int(n) for n in args["shape"].split(","))
    spacing = tuple(float(n) for n in args["spacing"].split(","))
    origin = tuple(float(n) for n in args["origin"].split(","))

    grid = landlab.graph.DualUniformRectilinearGraph(
        shape, spacing=spacing, origin=origin
    )
    return as_resource(
        to_resource(
            grid,
            href=urllib.parse.urlunsplit(
                ("", "", "/graphs/raster", urllib.parse.urlencode(args), "")
            ),
            repr_="DualUniformRectilinearGraph({shape}, spacing={spacing}, origin={origin})".format(
                shape=repr(shape), spacing=repr(spacing), origin=repr(origin)
            ),
        )
    )


@graphs_page.route("/hex")
def hex():
    args = dict(
        shape=request.args.get("shape", "4,4"),
        spacing=request.args.get("spacing", "1.0"),
        origin=request.args.get("origin", "0.0,0.0"),
        orientation=request.args.get("orientation", "horizontal"),
        node_layout=request.args.get("node_layout", "rect"),
    )

    shape = tuple(int(n) for n in args["shape"].split(","))
    spacing = float(args["spacing"])
    origin = tuple(float(n) for n in args["origin"].split(","))

    grid = landlab.graph.DualHexGraph(
        shape,
        spacing=spacing,
        origin=origin,
        orientation=args["orientation"],
        node_layout=args["node_layout"],
    )

    return as_resource(
        to_resource(
            grid,
            href=urllib.parse.urlunsplit(
                ("", "", "/graphs/hex", urllib.parse.urlencode(args), "")
            ),
            repr_="DualHexGraph({shape}, spacing={spacing}, origin={origin}, orientation={orientation}, node_layout={node_layout})".format(
                shape=repr(shape),
                spacing=repr(spacing),
                origin=repr(origin),
                orientation=repr(args["orientation"]),
                node_layout=repr(args["node_layout"]),
            ),
        )
    )


@graphs_page.route("/radial")
def radial():
    args = dict(
        shape=request.args.get("shape", "3,4"),
        spacing=request.args.get("spacing", "1.0"),
        origin=request.args.get("origin", "0.0,0.0"),
    )

    shape = tuple(int(n) for n in args["shape"].split(","))
    spacing = float(args["spacing"])
    origin = tuple(float(n) for n in args["origin"].split(","))

    grid = landlab.graph.DualRadialGraph(shape, spacing=spacing, origin=origin)

    return as_resource(
        to_resource(
            grid,
            href=urllib.parse.urlunsplit(
                ("", "", "/graphs/radial", urllib.parse.urlencode(args), "")
            ),
            repr_="DualRadialGraph({shape}, spacing={spacing}, origin={origin})".format(
                shape=repr(shape), spacing=repr(spacing), origin=repr(origin)
            ),
        )
    )



#! /usr/bin/env python
from setuptools import find_packages, setup

import versioneer


setup(
    name="landlab_rest",
    version=versioneer.get_version(),
    author="Eric Hutton",
    author_email="eric.hutton@colorado.edu",
    description="A RESTful interface to landlab graphs.",
    long_description=open("README.rst").read(),
    classifiers=[
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Cython",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Scientific/Engineering :: Physics",
    ],
    install_requires=["flask", "cherrypy", "click", "landlab", "xarray"],
    packages=find_packages(),
    cmdclass=versioneer.get_cmdclass(),
    entry_points={"console_scripts": ["start-sketchbook=landlab_rest.cli:main"]},
)
